/*
 * Copyright (c) 2019 AlexIIL
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
package alexiil.mc.lib.multipart.api.event;

import alexiil.mc.lib.multipart.api.property.MultipartProperties;
import alexiil.mc.lib.multipart.impl.LmpInternalOnly;
import net.minecraft.class_1922;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;

/** Fired during {@link class_2248#method_9603(class_2680, class_1922,class_2338, class_2350)} and
 * {@link class_2248#method_9524(class_2680, class_1922, class_2338, class_2350)}, however the more specific classes
 * {@link PartStrongRedstonePowerEvent} and {@link PartWeakRedstonePowerEvent} are only fired during their respective
 * method calls. */
public abstract class PartRedstonePowerEvent extends MultipartEvent {

    @LmpInternalOnly
    static final PartRedstonePowerEventFactory STRONG_FACTORY = PartStrongRedstonePowerEvent::new;

    @LmpInternalOnly
    static final PartRedstonePowerEventFactory WEAK_FACTORY = PartWeakRedstonePowerEvent::new;

    /** The value in {@link MultipartProperties#getWeakRedstonePower(class_2350)} (and
     * {@link MultipartProperties#getStrongRedstonePower(class_2350)} if this {@link #isStrong()}). */
    public final int powerProperty;

    /** @return The side of the block that is being */
    public final class_2350 side;

    @LmpInternalOnly
    int value;

    private PartRedstonePowerEvent(int powerProperty, class_2350 side) {
        this.powerProperty = powerProperty;
        this.side = side;
        this.value = powerProperty;
    }

    public void set(int value) {
        if (value < 0 || value >= 16) {
            throw new IllegalArgumentException("value out of bounds!");
        }

        this.value = Math.max(value, this.value);
    }

    public abstract boolean isStrong();

    public static final class PartStrongRedstonePowerEvent extends PartRedstonePowerEvent {

        private PartStrongRedstonePowerEvent(int powerProperty, class_2350 side) {
            super(powerProperty, side);
        }

        @Override
        public boolean isStrong() {
            return true;
        }
    }

    public static final class PartWeakRedstonePowerEvent extends PartRedstonePowerEvent {

        private PartWeakRedstonePowerEvent(int powerProperty, class_2350 side) {
            super(powerProperty, side);
        }

        @Override
        public boolean isStrong() {
            return false;
        }
    }

    /** Event factory. Not part of the public api, since using it means LMP can't add any fields in the future. */
    @FunctionalInterface
    public interface PartRedstonePowerEventFactory {
        PartRedstonePowerEvent create(int fromProperty, class_2350 side);
    }
}
