/*
 * Copyright (c) 2019 AlexIIL
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
package alexiil.mc.lib.multipart.impl;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import net.minecraft.class_2338;
import net.minecraft.class_2415;
import net.minecraft.class_2470;
import net.minecraft.class_2487;
import alexiil.mc.lib.multipart.api.AbstractPart;
import alexiil.mc.lib.multipart.api.MultipartHolder;

/** Combined {@link class_2338} + {@link PartHolder#uniqueId}. */
final class PosPartId {

    private static final Set<String> OLD_EXACT_TAG_NAMES = new HashSet<>(Arrays.asList("x", "y", "z", "u"));
    private static final Set<String> NEW_OFFSET_TAG_NAMES = new HashSet<>(Arrays.asList("a", "b", "c", "u"));

    final class_2338 pos;
    final long uid;

    PosPartId(class_2338 pos, long uid) {
        this.pos = pos.method_10062();
        this.uid = uid;
    }

    PosPartId(AbstractPart part) {
        this(part.holder);
    }

    PosPartId(MultipartHolder holder) {
        this(holder.getContainer().getMultipartPos(), holder.getUniqueId());
    }

    PosPartId(PartContainer from, class_2487 tag) {
        if (tag.method_10545("x")) {
            pos = new class_2338(tag.method_10550("x"), tag.method_10550("y"), tag.method_10550("z"));
        } else {
            class_2338 f = from.getMultipartPos();
            pos = new class_2338(tag.method_10550("a") + f.method_10263(), tag.method_10550("b") + f.method_10264(), tag.method_10550("c") + f.method_10260());
        }
        uid = tag.method_10537("u");
    }

    class_2487 toTag(PartContainer from) {
        class_2338 f = from.getMultipartPos();
        class_2487 tag = new class_2487();
        tag.method_10569("a", pos.method_10263() - f.method_10263());
        tag.method_10569("b", pos.method_10264() - f.method_10264());
        tag.method_10569("c", pos.method_10260() - f.method_10260());
        tag.method_10544("u", uid);
        return tag;
    }

    public static boolean isValid(class_2487 tag) {
        Set<String> keys = tag.method_10541();
        return keys.size() == 4 && (keys.containsAll(OLD_EXACT_TAG_NAMES) || keys.containsAll(NEW_OFFSET_TAG_NAMES));
    }

    @Override
    public String toString() {
        return "{ " + pos.method_10263() + ", " + pos.method_10264() + ", " + pos.method_10260() + " #" + uid + "}";
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (obj.getClass() != getClass()) {
            return false;
        }
        PosPartId other = (PosPartId) obj;
        return uid == other.uid && pos.equals(other.pos);
    }

    @Override
    public int hashCode() {
        // 524287 = 2 ^ 19 - 1
        long lval = (((uid * 524287 + pos.method_10263()) * 31) + pos.method_10264()) * 31 + pos.method_10260();
        return Long.hashCode(lval);
    }

    public boolean posEquals(class_2338 otherPos) {
        return pos.equals(otherPos);
    }

    public boolean isFor(PartHolder req) {
        return uid == req.uniqueId && posEquals(req.container.getMultipartPos());
    }

    public PosPartId rotate(PartContainer from, class_2470 rotation) {
        if (rotation == class_2470.field_11467) {
            return this;
        }
        class_2338 f = from.getMultipartPos();
        return new PosPartId(pos.method_10059(f).method_10070(rotation).method_10081(f), uid);
    }

    public PosPartId mirror(PartContainer from, class_2415 mirror) {
        if (mirror == class_2415.field_11302) {
            return this;
        }
        class_2338 f = from.getMultipartPos();
        int x = pos.method_10263() - f.method_10263();
        int z = pos.method_10260() - f.method_10260();
        if (mirror == class_2415.field_11300) {
            x = -x;
        } else {
            assert mirror == class_2415.field_11301;
            z = -z;
        }
        return new PosPartId(new class_2338(x + f.method_10263(), pos.method_10264(), z + f.method_10260()), uid);
    }
}
