/*
 * Copyright (c) 2019 AlexIIL
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
package alexiil.mc.lib.multipart.impl.client.model;

import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.function.Supplier;

import net.fabricmc.fabric.api.renderer.v1.model.FabricBakedModel;
import net.fabricmc.fabric.api.renderer.v1.render.RenderContext;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_1799;
import net.minecraft.class_1920;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_310;
import net.minecraft.class_777;
import net.minecraft.class_806;
import net.minecraft.class_809;
import alexiil.mc.lib.multipart.api.AbstractPart;
import alexiil.mc.lib.multipart.api.render.PartModelBaker;
import alexiil.mc.lib.multipart.api.render.PartModelKey;
import alexiil.mc.lib.multipart.impl.LibMultiPartClient;

/** A {@link class_1087} that wraps a single {@link AbstractPart}. */
public class SinglePartBakedModel<K extends PartModelKey> implements class_1087, FabricBakedModel {

    public final K key;
    public final Class<K> clazz;

    public SinglePartBakedModel(K key, Class<K> clazz) {
        this.key = key;
        this.clazz = clazz;
    }

    public static SinglePartBakedModel<?> create(PartModelKey key) {
        return key == null ? new SinglePartBakedModel<>(null, PartModelKey.class) : create0(key, key.getClass());
    }

    private static <K extends PartModelKey> SinglePartBakedModel<?> create0(PartModelKey key, Class<K> clazz) {
        return new SinglePartBakedModel<>(clazz.cast(key), clazz);
    }

    @Override
    public boolean isVanillaAdapter() {
        return false;
    }

    @Override
    public void emitBlockQuads(
        class_1920 blockView, class_2680 state, class_2338 pos, Supplier<Random> randomSupplier,
        RenderContext context
    ) {
        emitQuads(context, true, randomSupplier);
    }

    @Override
    public void emitItemQuads(class_1799 stack, Supplier<Random> randomSupplier, RenderContext context) {
        emitQuads(context, false, randomSupplier);
    }

    private void emitQuads(RenderContext context, boolean shouldQuadsBeLit, Supplier<Random> randomSupplier) {
        if (key == null) {
            return;
        }
        class_1087 model
            = class_310.method_1551().method_1554().method_4742(LibMultiPartClient.MODEL_IDENTIFIER);
        if (!(model instanceof MultipartModel)) {
            return;
        }
        PartModelBaker<? super K> baker = (PartModelBaker<? super K>) ((MultipartModel) model).getBaker(key.getClass());
        if (baker != null) {
            baker.emitQuads(key, new NormalPartRenderContext(context, shouldQuadsBeLit, randomSupplier));
        }
    }

    @Override
    public List<class_777> method_4707(class_2680 var1, class_2350 var2, Random var3) {
        return Collections.emptyList();
    }

    @Override
    public boolean method_4708() {
        return false;
    }

    @Override
    public boolean method_4712() {
        return false;
    }

    @Override
    public boolean method_24304() {
        return true;
    }

    @Override
    public boolean method_4713() {
        return false;
    }

    @Override
    public class_1058 method_4711() {
        return class_310.method_1551().method_1541().method_3351().method_3333().method_4744()
            .method_4711();
    }

    @Override
    public class_809 method_4709() {
        return class_809.field_4301;
    }

    @Override
    public class_806 method_4710() {
        return class_806.field_4292;
    }
}
