/*
 * Copyright (c) 2019 AlexIIL
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
package alexiil.mc.lib.multipart.impl.client.render;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.class_310;
import net.minecraft.class_3695;
import net.minecraft.class_4587;
import net.minecraft.class_4597;
import net.minecraft.class_5614;
import net.minecraft.class_827;
import alexiil.mc.lib.multipart.api.AbstractPart;
import alexiil.mc.lib.multipart.api.render.PartDynamicModelRegisterEvent;
import alexiil.mc.lib.multipart.api.render.PartRenderer;
import alexiil.mc.lib.multipart.impl.MultipartBlockEntity;
import alexiil.mc.lib.multipart.impl.PartHolder;

public class MultipartBlockEntityRenderer implements class_827<MultipartBlockEntity> {
    private final Map<Class<? extends AbstractPart>, PartRenderer<?>> renderers = new HashMap<>();
    private final Map<Class<? extends AbstractPart>, PartRenderData<?>> resolved = new HashMap<>();

    public MultipartBlockEntityRenderer(class_5614.class_5615 ctx) {
        PartDynamicModelRegisterEvent.EVENT.invoker().registerModels(new PartDynamicModelRegisterEvent.DynamicModelRenderer() {
            @Override
            public <P extends AbstractPart> void register(Class<P> clazz, PartRenderer<P> renderer) {
                renderers.put(clazz, renderer);
                resolved.clear();
            }
        });
    }

    private <P extends AbstractPart> PartRenderData<? super P> getRenderer(Class<P> clazz) {
        PartRenderData<?> renderer = resolved.get(clazz);
        if (renderer != null) {
            return (PartRenderData<? super P>) renderer;
        }

        Class<? super P> c = clazz;
        do {
            PartRenderer<?> pr = renderers.get(c);
            if (pr != null) {
                PartRenderData<?> data = new PartRenderData<>(pr, getDebugName(clazz));
                resolved.put(clazz, data);
                return (PartRenderData<? super P>) data;
            }
        } while ((c = c.getSuperclass()) != null);
        PartRenderData<AbstractPart> data = new PartRenderData<>(NoopRenderer.INSTANCE, getDebugName(clazz));
        resolved.put(clazz, data);
        return data;
    }

    private static String getDebugName(Class<?> clazz) {
        Package pkg = clazz.getPackage();
        String fqn = clazz.getName();

        if (pkg == null) {
            return fqn;
        }

        if (fqn.startsWith(pkg.getName() + ".")) {
            return fqn.substring(pkg.getName().length() + 1);
        } else {
            return fqn;
        }

    }

    @Override
    public void render(
        MultipartBlockEntity be, float tickDelta, class_4587 matrices, class_4597 vertexConsumers,
        int light, int overlay
    ) {
        class_3695 p = class_310.method_1551().method_16011();

        p.method_15396("LMP");
        for (PartHolder holder : be.getContainer().parts) {
            renderPart(p, holder.part, holder.part.getClass(), tickDelta, matrices, vertexConsumers, light, overlay);
        }
        p.method_15407();
    }

    <P extends AbstractPart> void renderPart(
        class_3695 p, AbstractPart part, Class<P> clazz, float tickDelta, class_4587 matrices,
        class_4597 vertexConsumers, int light, int overlay
    ) {
        PartRenderData<? super P> data = getRenderer(clazz);
        p.method_15396(data.debugName);
        data.renderer.render(clazz.cast(part), tickDelta, matrices, vertexConsumers, light, overlay);
        p.method_15407();
    }

    enum NoopRenderer implements PartRenderer<AbstractPart> {
        INSTANCE;

        @Override
        public void render(
            AbstractPart part, float tickDelta, class_4587 matrices, class_4597 vertexConsumers, int light,
            int overlay
        ) {
            // NO-OP
        }
    }

    static class PartRenderData<P extends AbstractPart> {
        final PartRenderer<P> renderer;
        final String debugName;

        PartRenderData(PartRenderer<P> renderer, String debugName) {
            this.renderer = renderer;
            this.debugName = debugName;
        }
    }
}
