/*
 * Copyright (c) 2019 AlexIIL
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
package alexiil.mc.lib.multipart.mixin.impl;

import java.util.Objects;
import net.minecraft.class_1657;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_239;
import net.minecraft.class_2680;
import net.minecraft.class_310;
import net.minecraft.class_636;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import alexiil.mc.lib.multipart.impl.LibMultiPart;
import alexiil.mc.lib.multipart.mixin.api.IBlockMultipart;
import alexiil.mc.lib.multipart.mixin.api.IClientPlayerInteractionManagerMixin;

@Mixin(class_636.class)
public class ClientPlayerInteractionManagerMixin implements IClientPlayerInteractionManagerMixin {

    @Shadow
    @Final
    class_310 client;

    @Shadow
    class_2338 currentBreakingPos;

    @Shadow
    private float blockBreakingSoundCooldown;

    @Shadow
    private float currentBreakingProgress;

    @Unique
    Object partKey;

    @Override
    public class_2338 libmultipart_getCurrentBreakPosition() {
        return currentBreakingPos;
    }

    @Override
    public Object libmultipart_getPartKey() {
        return partKey;
    }

    @Redirect(
        at = @At(value = "INVOKE",
            target = "Lnet/minecraft/block/BlockState;onBlockBreakStart(Lnet/minecraft/world/World;"
                + "Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/entity/player/PlayerEntity;)V"),
        method = "attackBlock(Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/math/Direction;)Z")
    void onBlockBreakStart(class_2680 state, class_1937 world, class_2338 pos, class_1657 player) {
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] onBlockBreakStart( " + pos + " " + state + " )");
        }
        class_2248 block = state.method_26204();
        if (block instanceof IBlockMultipart<?>) {
            IBlockMultipart<?> blockMulti = (IBlockMultipart<?>) block;
            onBlockBreakStart0(state, world, pos, player, blockMulti);
        } else {
            state.method_26179(world, pos, player);
        }
    }

    @Redirect(
        at = @At(
            value = "INVOKE", target = "Lnet/minecraft/block/BlockState;calcBlockBreakingDelta(Lnet/minecraft/entity/player/PlayerEntity;"
                + "Lnet/minecraft/world/BlockView;Lnet/minecraft/util/math/BlockPos;)F"
        ), method = "attackBlock(Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/math/Direction;)Z"
    )
    float calcBlockBreakingDeltaAttack(class_2680 state, class_1657 pl, class_1922 view, class_2338 pos) {
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] calcBlockBreakingDelta( " + pos + " " + state + " )");
        }
        if (state.method_26204() instanceof IBlockMultipart<?>) {
            IBlockMultipart<?> block = (IBlockMultipart<?>) state.method_26204();
            return calcBlockBreakingDelta0(block, state, pl, view, pos, false);
        } else {
            return state.method_26165(pl, view, pos);
        }
    }

    @Redirect(
        at = @At(
            value = "INVOKE", target = "Lnet/minecraft/block/BlockState;calcBlockBreakingDelta(Lnet/minecraft/entity/player/PlayerEntity;"
                + "Lnet/minecraft/world/BlockView;Lnet/minecraft/util/math/BlockPos;)F"
        ), method = "updateBlockBreakingProgress(Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/math/Direction;)Z"
    )
    float calcBlockBreakingDeltaUpdate(class_2680 state, class_1657 pl, class_1922 view, class_2338 pos) {
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] calcBlockBreakingDelta( " + pos + " " + state + " )");
        }
        if (state.method_26204() instanceof IBlockMultipart<?>) {
            IBlockMultipart<?> block = (IBlockMultipart<?>) state.method_26204();
            return calcBlockBreakingDelta0(block, state, pl, view, pos, true);
        } else {
            return state.method_26165(pl, view, pos);
        }
    }

    private <T> float calcBlockBreakingDelta0(
        IBlockMultipart<T> block, class_2680 state, class_1657 pl, class_1922 view, class_2338 pos, boolean playHitSound
    ) {
        if (partKey == null) {
            return state.method_26165(pl, view, pos);
        }

        if (!block.getKeyClass().isInstance(partKey)) {
            if (LibMultiPart.DEBUG) {
                LibMultiPart.LOGGER.info(
                    "[player-interaction] calcBlockBreakingDelta0(): Wrong key " + partKey.getClass() + ", expected "
                        + block.getKeyClass()
                );
            }
            return state.method_26165(pl, view, pos);
        }

        T key = block.getKeyClass().cast(partKey);
        float delta = block.calcBlockBreakingDelta(state, pl, view, pos, key);
        if (playHitSound) {
            boolean reallyPlay = blockBreakingSoundCooldown % 4 == 0 || blockBreakingSoundCooldown == 4.1f;
            if (reallyPlay && block.playHitSound(pl.field_6002, pos, state, pl, key)) {
                blockBreakingSoundCooldown = 0.1f;
            }
        }
        return delta;
    }

    private <T> void onBlockBreakStart0(
        class_2680 state, class_1937 world, class_2338 pos, class_1657 player, IBlockMultipart<T> blockMulti
    ) {

        class_239 hit = class_310.method_1551().field_1765;
        T key = blockMulti.getTargetedMultipart(state, world, pos, hit.method_17784());
        partKey = key;
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] onBlockBreakStart0(): hit = " + hit);
            LibMultiPart.LOGGER.info("[player-interaction] onBlockBreakStart0(): key = " + key);
        }
        blockMulti.onBlockBreakStart(state, world, pos, player, key);
    }

    @Inject(method = "breakBlock(Lnet/minecraft/util/math/BlockPos;)Z",
        at = @At(value = "INVOKE",
            target = "Lnet/minecraft/block/Block;onBreak(Lnet/minecraft/world/World;Lnet/minecraft/util/math/BlockPos;"
                + "Lnet/minecraft/block/BlockState;Lnet/minecraft/entity/player/PlayerEntity;)V"),
        cancellable = true)
    void breakBlock(class_2338 pos, CallbackInfoReturnable<Boolean> ci) {
        class_1937 world = client.field_1687;
        class_2680 state = world.method_8320(pos);
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] breakBlock( " + pos + " " + state + " )");
        }
        if (state.method_26204() instanceof IBlockMultipart<?>) {
            IBlockMultipart<?> blockMulti = (IBlockMultipart<?>) state.method_26204();
            Boolean ret = breakBlock0(pos, blockMulti);
            if (ret != null) {
                ci.setReturnValue(ret);
                return;
            }
        }
    }

    private <T> Boolean breakBlock0(class_2338 pos, IBlockMultipart<T> blockMulti) {

        class_239 hit = class_310.method_1551().field_1765;
        class_1937 world = client.field_1687;
        class_2680 state = world.method_8320(pos);
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] breakBlock0(): hit = " + hit);
        }
        T target = blockMulti.getTargetedMultipart(state, world, pos, hit.method_17784());
        T previous;
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] breakBlock0(): target = " + target);
        }
        if (partKey == null) {
            previous = target;
        } else if (blockMulti.getKeyClass().isInstance(partKey)) {
            previous = blockMulti.getKeyClass().cast(partKey);
        } else {
            previous = target;
        }
        if (LibMultiPart.DEBUG) {
            LibMultiPart.LOGGER.info("[player-interaction] breakBlock0(): previous = " + previous);
        }
        partKey = null;
        if (target == null || !Objects.equals(previous, target)) {
            if (LibMultiPart.DEBUG) {
                LibMultiPart.LOGGER.info(
                    "[player-interaction] breakBlock0(): Different subpart keys: previous = " + previous
                        + ", current = " + target
                );
            }
            currentBreakingPos = new class_2338(currentBreakingPos.method_10263(), -1, currentBreakingPos.method_10260());
            return Boolean.FALSE;
        }
        blockMulti.onBreak(world, pos, state, client.field_1724, target);
        if (blockMulti.clearBlockState(world, pos, target)) {
            blockMulti.onBroken(world, pos, state, target);
            currentBreakingPos = new class_2338(currentBreakingPos.method_10263(), -1, currentBreakingPos.method_10260());
            return Boolean.TRUE;
        } else {
            currentBreakingPos = new class_2338(currentBreakingPos.method_10263(), -1, currentBreakingPos.method_10260());
            return Boolean.FALSE;
        }
    }
}
